import java.util.*;
import java.io.*;

public class Datenliste {
  protected ArrayList<Datenelement> daten;
  protected int anzahlElemente;
  protected String[] merkmale;      // Namen der Merkmale 
  protected String[] klassen;       // Namen der Klassen 
  protected int anzahlKlassen; 
  
  // Konstruktor: Leere Datenliste mit vorgegebenen Merkmalen und Klassen 
  public Datenliste(String[] merkmale, String[] klassen) {
    daten = new ArrayList<Datenelement>();
    anzahlElemente = 0;
    this.merkmale = merkmale;
    this.klassen = klassen;  
    anzahlKlassen = klassen.length;
  }                                                                                                                                                                                       
  
  public int getAnzahlElemente() {
    return anzahlElemente;
  }
  
  public int getAnzahlMerkmale() {
    return merkmale.length;
  }
  
  public Datenelement getElement(int i) {
    return daten.get(i);
  }
  
  public String getMerkmal(int i) {
    return merkmale[i];
  }
  
  // liefert Teilliste derjenigen Daten, fr die merkmal = wert gilt.     
  public Datenliste erzeugeTeilliste(int merkmal, int wert) {
    Datenliste teilliste = new Datenliste(merkmale, klassen);
    for (int i=0; i<anzahlElemente; i++) {
      Datenelement aktuell = getElement(i);
      if (aktuell.getAuspraegung(merkmal) == wert) {
        teilliste.daten.add(aktuell);
        teilliste.anzahlElemente++; 
      }
    }
    return teilliste;
  }  
  
  // bestimmt die hufigste Klasse 
  public String haeufigsteKlasse() {
    int[] counter = new int[anzahlKlassen];
    for (int i=0; i<anzahlElemente; i++) {
      for (int k=0; k<anzahlKlassen; k++) {
        if (getElement(i).getKlasse().equals(klassen[k])) {
          counter[k]++;
        }    
      }
    } 
    int max = -1;
    String result = "";
    for (int k=0; k<anzahlKlassen; k++) {
      if (counter[k] > max) {
        result = klassen[k];
        max = counter[k];
      }
    }
    return result;
  }
  
  // berprft, ob nur Datenelemente einer Klasse vorkommen
  public boolean istHomogen() {
    boolean result = true;
    if (anzahlElemente > 1) {
      String klasse = getElement(0).getKlasse();
      for(int i=1; i < anzahlElemente; i++) {
        if (!getElement(i).getKlasse().equals(klasse)) {
          result = false;
        }
      }
    }
    return result;
  }
  
  // berprft, ob nur Datenelemente mit gleichen Merkmalsausprgungen vorkommen
  public boolean gleicheAuspraegungen() {
    boolean result = true;
    if (anzahlElemente > 1) {
      Datenelement erstes = getElement(0);
      for(int i=1; i < anzahlElemente; i++) {
        if (!getElement(i).identischMit(erstes)) {
          result = false;
        }
      }
    }
    return result;
  }
  
  // Methode zur Bestimmung der FKR bei einem Split nach Merkmal m 
  public double berechneFKR(int m) {
    // Aufteilung der Datenelemente in zwei Teillisten
    Datenliste[] teilliste = new Datenliste[2];
    teilliste[0] = erzeugeTeilliste(m,0);
    teilliste[1] = erzeugeTeilliste(m,1);
    double falsch = 0;
    for (int j=0; j<2; j++) {
      // Die Wahl eines Merkmals, in dem alle Datenelemente bereinstimmen,
      // wird verhindert 
      if (teilliste[j].getAnzahlElemente() == this.getAnzahlElemente()) {
        falsch = this.getAnzahlElemente()+1;
      }
      else {
        String klasse = teilliste[j].haeufigsteKlasse(); 
        for (int e=0; e<teilliste[j].getAnzahlElemente(); e++) {
          if (!teilliste[j].getElement(e).getKlasse().equals(klasse)) {
            falsch++;   
          }
        }
      }
    }
    return falsch/this.getAnzahlElemente();
  }
  
  // Methoden zum Einlesen der Daten aus einer CSV-Datei 
  public Datenliste(String dateiname) {
    BufferedReader br = null;
    try {
      ArrayList<String[]> zeilen = new ArrayList<String[]>();
      br = new BufferedReader(new FileReader(new File(dateiname)));
      String zeile = br.readLine();
      // Einlesen der Klassen- und Merkmalsbezeichner 
      klassen = zeile.split(",");
      anzahlKlassen = klassen.length; 
      zeile = br.readLine();
      merkmale = zeile.split(",");
      zeile = br.readLine();
      while (zeile != null) {
        zeilen.add(zeile.split(","));
        zeile = br.readLine();
      } 
      br.close();
      if (zeilen.size() > 0) {
        daten = new ArrayList<Datenelement>();
        anzahlElemente = 0;
        for (int i=0; i<zeilen.size(); i++) {     
          daten.add(new Datenelement(zeilen.get(i)));
          anzahlElemente++;
        }
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  } 
  
  // Ausgabe
  public String toString() {
    String ergebnis = "";
    if (anzahlElemente > 0) {
      ergebnis += "("+getMerkmal(0); 
      for (int i=1; i<getAnzahlMerkmale(); i++) {
        ergebnis += "|"+getMerkmal(i); 
      }
      ergebnis += ")"+System.getProperty("line.separator");
      for (int i=0; i<anzahlElemente; i++) {
        ergebnis += getElement(i).toString() + System.getProperty("line.separator");
      }
    }
    return ergebnis;
  }
  
}